@@ -1,6 +1,5 @@
-import glob
-import os
 import sys
+from pathlib import Path
 from typing import Optional
 
 import cffi  # type: ignore
@@ -9,46 +8,51 @@ from . import hostfxr, mono, netfx
 
 __all__ = ["ffi", "load_hostfxr", "load_mono", "load_netfx"]
 
-ffi = cffi.FFI()
+ffi = cffi.FFI()  # type: ignore
 
 for cdef in hostfxr.cdef + mono.cdef + netfx.cdef:
     ffi.cdef(cdef)
 
 
-def load_hostfxr(dotnet_root: str):
+def load_hostfxr(dotnet_root: Path):
     hostfxr_name = _get_dll_name("hostfxr")
-    hostfxr_path = os.path.join(dotnet_root, "host", "fxr", "?.*", hostfxr_name)
 
-    for hostfxr_path in reversed(sorted(glob.glob(hostfxr_path))):
+    # This will fail as soon as .NET hits version 10, but hopefully by then
+    # we'll have a more robust way of finding the libhostfxr
+    hostfxr_path = dotnet_root / "host" / "fxr"
+    hostfxr_paths = hostfxr_path.glob(f"?.*/{hostfxr_name}")
+
+    for hostfxr_path in reversed(sorted(hostfxr_paths)):
         try:
-            return ffi.dlopen(hostfxr_path)
+            return ffi.dlopen(str(hostfxr_path))
         except Exception:
             pass
 
     raise RuntimeError(f"Could not find a suitable hostfxr library in {dotnet_root}")
 
 
-def load_mono(path: Optional[str] = None, gc: Optional[str] = None):
+def load_mono(path: Optional[Path] = None):
     # Preload C++ standard library, Mono needs that and doesn't properly link against it
-    if sys.platform.startswith("linux"):
+    if sys.platform == "linux":
         ffi.dlopen("stdc++", ffi.RTLD_GLOBAL)
 
-    return ffi.dlopen(path, ffi.RTLD_GLOBAL)
+    path_str = str(path) if path else None
+    return ffi.dlopen(path_str, ffi.RTLD_GLOBAL)
 
 
 def load_netfx():
     if sys.platform != "win32":
         raise RuntimeError(".NET Framework is only supported on Windows")
 
-    dirname = os.path.join(os.path.dirname(__file__), "dlls")
-    if sys.maxsize > 2 ** 32:
+    dirname = Path(__file__).parent / "dlls"
+    if sys.maxsize > 2**32:
         arch = "amd64"
     else:
         arch = "x86"
 
-    path = os.path.join(dirname, arch, "ClrLoader.dll")
+    path = dirname / arch / "ClrLoader.dll"
 
-    return ffi.dlopen(path)
+    return ffi.dlopen(str(path))
 
 
 def _get_dll_name(name: str) -> str:
